<input type="text" name="{{name}}" id="{{html_id}}" value="{{current_repr}}" autocomplete="off" {{ extra_attrs }} data-bob-text-field-options='{{ plugin_options }}' />
{% block help %}{% if help_text %}<p class="help">{{ help_text }}</p>{% endif %}{% endblock %}
